@@ -1,3 +1,6 @@
 """Atomate2 is a library of computational materials science workflows."""
 
 from atomate2._version import __version__
+from atomate2.settings import Atomate2Settings
+
+SETTINGS = Atomate2Settings()
