@@ -12,6 +12,7 @@ from pymatgen.core.structure import Structure
 
 from atomate2 import SETTINGS
 from atomate2.amset.jobs import AmsetMaker
+from atomate2.vasp.flows.core import DoubleRelaxMaker
 from atomate2.vasp.flows.elastic import ElasticMaker
 from atomate2.vasp.jobs.amset import (
     DenseUniformMaker,
@@ -32,6 +33,7 @@ from atomate2.vasp.jobs.core import (
     StaticMaker,
     TightRelaxMaker,
 )
+from atomate2.vasp.sets.core import HSEBSSetGenerator
 
 __all__ = ["VaspAmsetMaker", "DeformationPotentialMaker", "HSEVaspAmsetMaker"]
 
@@ -100,6 +102,7 @@ class DeformationPotentialMaker(Maker):
         bulk = self.static_deformation_maker.make(
             structure, prev_vasp_dir=prev_vasp_dir
         )
+        bulk.append_name("bulk ", prepend=True)
 
         # all deformation calculations need to be on the same k-point mesh, to achieve
         # this we override user_kpoints_settings with the desired k-points
@@ -125,7 +128,7 @@ class DeformationPotentialMaker(Maker):
 
         return Flow(
             jobs=[bulk, vasp_deformation_calcs, deformation_potentials],
-            output=deformation_potentials.output["dir_name"],
+            output=deformation_potentials.output,
             name=self.name,
         )
 
@@ -176,16 +179,24 @@ class VaspAmsetMaker(Maker):
     temperatures: tuple[float, ...] = _DEFAULT_TEMPERATURES
     use_hse_gap: bool = True
     amset_settings: dict = field(default_factory=dict)
-    relax_maker: BaseVaspMaker | None = field(default_factory=TightRelaxMaker)
+    relax_maker: BaseVaspMaker | None = field(
+        default_factory=lambda: DoubleRelaxMaker(relax_maker=TightRelaxMaker())
+    )
     static_maker: BaseVaspMaker = field(default_factory=StaticMaker)
     dense_uniform_maker: BaseVaspMaker = field(default_factory=DenseUniformMaker)
     dielectric_maker: BaseVaspMaker = field(default_factory=DielectricMaker)
-    elastic_maker: ElasticMaker = field(default_factory=ElasticMaker)
+    elastic_maker: ElasticMaker = field(
+        default_factory=lambda: ElasticMaker(bulk_relax_maker=None)
+    )
     deformation_potential_maker: DeformationPotentialMaker = field(
         default_factory=DeformationPotentialMaker
     )
-    hse_gap_maker: BaseVaspMaker = field(default_factory=HSEBSMaker)
-    amset_maker: AmsetMaker = field(default_factory=AmsetMaker)
+    hse_gap_maker: BaseVaspMaker = field(
+        default_factory=lambda: HSEBSMaker(
+            input_set_generator=HSEBSSetGenerator(user_incar_settings={"EDIFF": 1e-5})
+        )
+    )
+    amset_maker: AmsetMaker = field(default_factory=lambda: AmsetMaker(resubmit=True))
 
     def make(
         self,
@@ -234,8 +245,8 @@ class VaspAmsetMaker(Maker):
         phonon_frequency = calculate_polar_phonon_frequency(
             dielectric.output.structure,
             dielectric.output.calcs_reversed[0].output.normalmode_frequencies,
-            dielectric.output.calcs_reversed[0].output.normalmode_eigenvectors,
-            dielectric.output.calcs_reversed[0].output.outcar.born,
+            dielectric.output.calcs_reversed[0].output.normalmode_eigenvecs,
+            dielectric.output.calcs_reversed[0].output.outcar["born"],
         )
 
         # wavefunction coefficients
@@ -252,8 +263,11 @@ class VaspAmsetMaker(Maker):
         # note: the naming of dielectric constants in VASP and pymatgen is wrong
         high_freq_dielectric = dielectric.output.calcs_reversed[0].output.epsilon_static
         static_dielectric = job(np.sum)(
-            dielectric.output.calcs_reversed[0].output.epsilon_ionic,
-            high_freq_dielectric,
+            [
+                dielectric.output.calcs_reversed[0].output.epsilon_ionic,
+                high_freq_dielectric,
+            ],
+            axis=0,
         )
 
         jobs += [
@@ -271,12 +285,15 @@ class VaspAmsetMaker(Maker):
         # set doping and temperature but be careful not to override user selections
         settings = {
             "doping": self.doping,
-            "temperature": self.temperatures,
-            "phonon_frequency": phonon_frequency.output["frequency"],
+            "temperatures": self.temperatures,
+            "pop_frequency": phonon_frequency.output["frequency"],
             "elastic_constant": elastic.output.elastic_tensor.raw,
             "high_frequency_dielectric": high_freq_dielectric,
             "static_dielectric": static_dielectric.output,
-            "deformation": "deformation.h5",
+            "deformation_potential": "deformation.h5",
+            "print_log": False,
+            "interpolation_factor": 5,
+            "free_carrier_screening": True,
         }
 
         if self.use_hse_gap and "bandgap" not in self.amset_settings:
@@ -296,6 +313,7 @@ class VaspAmsetMaker(Maker):
             settings,
             wavefunction_dir=wavefunction.output["dir_name"],
             deformation_dir=deformation.output["dir_name"],
+            bandstructure_dir=dense_bs.output.dir_name,
         )
         jobs.append(amset)
 
@@ -346,7 +364,9 @@ class HSEVaspAmsetMaker(Maker):
     doping: tuple[float, ...] = _DEFAULT_DOPING
     temperatures: tuple[float, ...] = _DEFAULT_TEMPERATURES
     amset_settings: dict = field(default_factory=dict)
-    relax_maker: BaseVaspMaker | None = field(default_factory=HSETightRelaxMaker)
+    relax_maker: BaseVaspMaker | None = field(
+        default_factory=lambda: DoubleRelaxMaker(relax_maker=HSETightRelaxMaker())
+    )
     static_maker: BaseVaspMaker = field(default_factory=HSEStaticMaker)
     dense_uniform_maker: BaseVaspMaker = field(default_factory=HSEDenseUniformMaker)
     deformation_potential_maker: DeformationPotentialMaker = field(
@@ -355,8 +375,10 @@ class HSEVaspAmsetMaker(Maker):
         )
     )
     dielectric_maker: BaseVaspMaker = field(default_factory=DielectricMaker)
-    elastic_maker: ElasticMaker = field(default_factory=ElasticMaker)
-    amset_maker: AmsetMaker = field(default_factory=AmsetMaker)
+    elastic_maker: ElasticMaker = field(
+        default_factory=lambda: ElasticMaker(bulk_relax_maker=None)
+    )
+    amset_maker: AmsetMaker = field(default_factory=lambda: AmsetMaker(resubmit=True))
 
     def make(
         self,
@@ -405,8 +427,8 @@ class HSEVaspAmsetMaker(Maker):
         phonon_frequency = calculate_polar_phonon_frequency(
             dielectric.output.structure,
             dielectric.output.calcs_reversed[0].output.normalmode_frequencies,
-            dielectric.output.calcs_reversed[0].output.normalmode_eigenvectors,
-            dielectric.output.calcs_reversed[0].output.outcar.born,
+            dielectric.output.calcs_reversed[0].output.normalmode_eigenvecs,
+            dielectric.output.calcs_reversed[0].output.outcar["born"],
         )
 
         # wavefunction coefficients
@@ -423,8 +445,11 @@ class HSEVaspAmsetMaker(Maker):
         # note: the naming of dielectric constants in VASP and pymatgen is wrong
         high_freq_dielectric = dielectric.output.calcs_reversed[0].output.epsilon_static
         static_dielectric = job(np.sum)(
-            dielectric.output.calcs_reversed[0].output.epsilon_ionic,
-            high_freq_dielectric,
+            [
+                dielectric.output.calcs_reversed[0].output.epsilon_ionic,
+                high_freq_dielectric,
+            ],
+            axis=0,
         )
 
         jobs += [
@@ -442,12 +467,15 @@ class HSEVaspAmsetMaker(Maker):
         # set doping and temperature but be careful not to override user selections
         settings = {
             "doping": self.doping,
-            "temperature": self.temperatures,
-            "phonon_frequency": phonon_frequency.output["frequency"],
+            "temperatures": self.temperatures,
+            "pop_frequency": phonon_frequency.output["frequency"],
             "elastic_constant": elastic.output.elastic_tensor.raw,
             "high_frequency_dielectric": high_freq_dielectric,
             "static_dielectric": static_dielectric.output,
-            "deformation": "deformation.h5",
+            "deformation_potential": "deformation.h5",
+            "print_log": False,
+            "interpolation_factor": 5,
+            "free_carrier_screening": True,
         }
 
         # apply the user settings
@@ -458,6 +486,7 @@ class HSEVaspAmsetMaker(Maker):
             settings,
             wavefunction_dir=wavefunction.output["dir_name"],
             deformation_dir=deformation.output["dir_name"],
+            bandstructure_dir=dense_bs.output.dir_name,
         )
         jobs.append(amset)
 
