@@ -6,6 +6,8 @@ import logging
 from dataclasses import dataclass, field
 from pathlib import Path
 
+from pymatgen.alchemy.materials import TransformedStructure
+from pymatgen.alchemy.transmuters import StandardTransmuter
 from pymatgen.core.structure import Structure
 
 from atomate2.vasp.jobs.base import BaseVaspMaker, vasp_job
@@ -33,6 +35,7 @@ __all__ = [
     "HSEStaticMaker",
     "TightRelaxMaker",
     "HSETightRelaxMaker",
+    "TransmuterMaker",
 ]
 
 
@@ -59,7 +62,10 @@ class StaticMaker(BaseVaspMaker):
         Keyword arguments that will get passed to :obj:`.should_stop_children`.
     write_additional_data : dict
         Additional data to write to the current directory. Given as a dict of
-        {filename: data}.
+        {filename: data}. Note that if using FireWorks, dictionary keys cannot contain
+        the "." character which is typically used to denote file extensions. To avoid
+        this, use the ":" character, which will automatically be converted to ".". E.g.
+        ``{"my_file:txt": "contents of the file"}``.
     """
 
     name: str = "static"
@@ -91,7 +97,10 @@ class RelaxMaker(BaseVaspMaker):
         Keyword arguments that will get passed to :obj:`.should_stop_children`.
     write_additional_data : dict
         Additional data to write to the current directory. Given as a dict of
-        {filename: data}.
+        {filename: data}. Note that if using FireWorks, dictionary keys cannot contain
+        the "." character which is typically used to denote file extensions. To avoid
+        this, use the ":" character, which will automatically be converted to ".". E.g.
+        ``{"my_file:txt": "contents of the file"}``.
     """
 
     name: str = "relax"
@@ -123,7 +132,10 @@ class TightRelaxMaker(BaseVaspMaker):
         Keyword arguments that will get passed to :obj:`.should_stop_children`.
     write_additional_data : dict
         Additional data to write to the current directory. Given as a dict of
-        {filename: data}.
+        {filename: data}. Note that if using FireWorks, dictionary keys cannot contain
+        the "." character which is typically used to denote file extensions. To avoid
+        this, use the ":" character, which will automatically be converted to ".". E.g.
+        ``{"my_file:txt": "contents of the file"}``.
     """
 
     name: str = "tight relax"
@@ -155,7 +167,10 @@ class NonSCFMaker(BaseVaspMaker):
         Keyword arguments that will get passed to :obj:`.should_stop_children`.
     write_additional_data : dict
         Additional data to write to the current directory. Given as a dict of
-        {filename: data}.
+        {filename: data}. Note that if using FireWorks, dictionary keys cannot contain
+        the "." character which is typically used to denote file extensions. To avoid
+        this, use the ":" character, which will automatically be converted to ".". E.g.
+        ``{"my_file:txt": "contents of the file"}``.
     """
 
     name: str = "non-scf"
@@ -223,7 +238,10 @@ class HSERelaxMaker(BaseVaspMaker):
         Keyword arguments that will get passed to :obj:`.should_stop_children`.
     write_additional_data : dict
         Additional data to write to the current directory. Given as a dict of
-        {filename: data}.
+        {filename: data}. Note that if using FireWorks, dictionary keys cannot contain
+        the "." character which is typically used to denote file extensions. To avoid
+        this, use the ":" character, which will automatically be converted to ".". E.g.
+        ``{"my_file:txt": "contents of the file"}``.
     """
 
     name: str = "hse relax"
@@ -251,9 +269,12 @@ class HSETightRelaxMaker(BaseVaspMaker):
         Keyword arguments that will get passed to :obj:`.TaskDocument.from_directory`.
     stop_children_kwargs
         Keyword arguments that will get passed to :obj:`.should_stop_children`.
-    write_additional_data
+    write_additional_data : dict
         Additional data to write to the current directory. Given as a dict of
-        {filename: data}.
+        {filename: data}. Note that if using FireWorks, dictionary keys cannot contain
+        the "." character which is typically used to denote file extensions. To avoid
+        this, use the ":" character, which will automatically be converted to ".". E.g.
+        ``{"my_file:txt": "contents of the file"}``.
     """
 
     name: str = "hse tight relax"
@@ -285,7 +306,10 @@ class HSEStaticMaker(BaseVaspMaker):
         Keyword arguments that will get passed to :obj:`.should_stop_children`.
     write_additional_data : dict
         Additional data to write to the current directory. Given as a dict of
-        {filename: data}.
+        {filename: data}. Note that if using FireWorks, dictionary keys cannot contain
+        the "." character which is typically used to denote file extensions. To avoid
+        this, use the ":" character, which will automatically be converted to ".". E.g.
+        ``{"my_file:txt": "contents of the file"}``.
     """
 
     name: str = "hse static"
@@ -299,6 +323,13 @@ class HSEBSMaker(BaseVaspMaker):
     """
     Maker to create HSE06 band structure jobs.
 
+    .. warning::
+        The number of bands will automatically be adjusted based on the number of bands
+        in the previous calculation. Therefore, if starting from a previous structure
+        ensure you are starting from a static/relaxation calculation that has the same
+        number of atoms (i.e., not a smaller/larger cell), as otherwise the number of
+        bands may be set incorrectly.
+
     Parameters
     ----------
     name : str
@@ -317,7 +348,10 @@ class HSEBSMaker(BaseVaspMaker):
         Keyword arguments that will get passed to :obj:`.should_stop_children`.
     write_additional_data : dict
         Additional data to write to the current directory. Given as a dict of
-        {filename: data}.
+        {filename: data}. Note that if using FireWorks, dictionary keys cannot contain
+        the "." character which is typically used to denote file extensions. To avoid
+        this, use the ":" character, which will automatically be converted to ".". E.g.
+        ``{"my_file:txt": "contents of the file"}``.
     """
 
     name: str = "hse band structure"
@@ -384,6 +418,10 @@ class DielectricMaker(BaseVaspMaker):
         The input structure should be well relaxed to avoid imaginary modes. For
         example, using :obj:`TightRelaxMaker`.
 
+    .. Note::
+        If starting from a previous calculation, magnetism will be disabled if all
+        MAGMOMs are less than 0.02.
+
     Parameters
     ----------
     name : str
@@ -402,10 +440,127 @@ class DielectricMaker(BaseVaspMaker):
         Keyword arguments that will get passed to :obj:`.should_stop_children`.
     write_additional_data : dict
         Additional data to write to the current directory. Given as a dict of
-        {filename: data}.
+        {filename: data}. Note that if using FireWorks, dictionary keys cannot contain
+        the "." character which is typically used to denote file extensions. To avoid
+        this, use the ":" character, which will automatically be converted to ".". E.g.
+        ``{"my_file:txt": "contents of the file"}``.
     """
 
     name: str = "dielectric"
     input_set_generator: StaticSetGenerator = field(
-        default_factory=lambda: StaticSetGenerator(lepsilon=True)
+        default_factory=lambda: StaticSetGenerator(lepsilon=True, auto_ispin=True)
     )
+
+
+@dataclass
+class TransmuterMaker(BaseVaspMaker):
+    """
+    A maker to apply transformations to a structure before writing the input sets.
+
+    Note that if a transformation yields many structures, only the last structure in the
+    list is used.
+
+    Parameters
+    ----------
+    name : str
+        The job name.
+    transformations : tuple of str
+        The transformations to apply. Given as a list of names of transformation classes
+        as defined in the modules in pymatgen.transformations. For example,
+        ``['DeformStructureTransformation', 'SupercellTransformation']``.
+    transformation_params : tuple of dict or None
+        The parameters used to instantiate each transformation class. Given as a list of
+        dicts.
+    input_set_generator : StaticSetGenerator
+        A generator used to make the input set.
+    write_input_set_kwargs : dict
+        Keyword arguments that will get passed to :obj:`.write_vasp_input_set`.
+    copy_vasp_kwargs : dict
+        Keyword arguments that will get passed to :obj:`.copy_vasp_outputs`.
+    run_vasp_kwargs : dict
+        Keyword arguments that will get passed to :obj:`.run_vasp`.
+    task_document_kwargs : dict
+        Keyword arguments that will get passed to :obj:`.TaskDocument.from_directory`.
+    stop_children_kwargs : dict
+        Keyword arguments that will get passed to :obj:`.should_stop_children`.
+    write_additional_data : dict
+        Additional data to write to the current directory. Given as a dict of
+        {filename: data}. Note that if using FireWorks, dictionary keys cannot contain
+        the "." character which is typically used to denote file extensions. To avoid
+        this, use the ":" character, which will automatically be converted to ".". E.g.
+        ``{"my_file:txt": "contents of the file"}``.
+    """
+
+    name: str = "transmuter"
+    transformations: tuple[str, ...] = field(default_factory=tuple)
+    transformation_params: tuple[dict, ...] | None = None
+    input_set_generator: VaspInputSetGenerator = field(
+        default_factory=StaticSetGenerator
+    )
+
+    @vasp_job
+    def make(
+        self,
+        structure: Structure,
+        prev_vasp_dir: str | Path | None = None,
+    ):
+        """
+        Run a transmuter VASP job.
+
+        Parameters
+        ----------
+        structure : Structure
+            A pymatgen structure object.
+        prev_vasp_dir : str or Path or None
+            A previous VASP calculation directory to copy output files from.
+        """
+        transformations = _get_transformations(
+            self.transformations, self.transformation_params
+        )
+        ts = TransformedStructure(structure)
+        transmuter = StandardTransmuter([ts], transformations)
+        structure = transmuter.transformed_structures[-1].final_structure
+
+        # to avoid mongoDB errors, ":" is automatically converted to "."
+        if "transformations:json" not in self.write_additional_data:
+            tjson = transmuter.transformed_structures[-1]
+            self.write_additional_data["transformations:json"] = tjson
+
+        return super().make.original(self, structure, prev_vasp_dir)
+
+
+def _get_transformations(
+    transformations: tuple[str, ...], params: tuple[dict, ...] | None
+):
+    """Get instantiated transformation objects from their names and parameters."""
+    params = ({},) * len(transformations) if params is None else params
+
+    if len(params) != len(transformations):
+        raise ValueError("Number of transformations and parameters must be the same.")
+
+    transformation_objects = []
+    for transformation, transformation_params in zip(transformations, params):
+        found = False
+        for m in (
+            "advanced_transformations",
+            "defect_transformations",
+            "site_transformations",
+            "standard_transformations",
+        ):
+            from importlib import import_module
+
+            mod = import_module(f"pymatgen.transformations.{m}")
+
+            try:
+                t_cls = getattr(mod, transformation)
+                found = True
+                continue
+            except AttributeError:
+                pass
+
+        if not found:
+            raise ValueError(f"Could not find transformation: {transformation}")
+
+        t_obj = t_cls(**transformation_params)
+        transformation_objects.append(t_obj)
+    return transformation_objects
