@@ -12,6 +12,8 @@ from pydantic.datetime_parse import datetime
 from pymatgen.command_line.bader_caller import bader_analysis_from_path
 from pymatgen.core.lattice import Lattice
 from pymatgen.core.structure import Structure
+from pymatgen.electronic_structure.bandstructure import BandStructure
+from pymatgen.electronic_structure.dos import Dos
 from pymatgen.io.vasp import (
     BSVasprun,
     Locpot,
@@ -39,6 +41,7 @@ __all__ = [
     "Status",
     "VaspObject",
     "PotcarSpec",
+    "FrequencyDependentDielectric",
     "CalculationInput",
     "CalculationOutput",
     "RunStatistics",
@@ -207,7 +210,70 @@ class RunStatistics(BaseModel):
             "Total CPU time used (sec)": "total_time",
             "cores": "cores",
         }
-        return cls(**{v: outcar.run_stats.get(k) or 0 for k, v in mapping.items()})
+        run_stats = {}
+        for k, v in mapping.items():
+            stat = outcar.run_stats.get(k) or 0
+            try:
+                stat = float(stat)
+            except ValueError:
+                # sometimes the statistics are misformatted
+                stat = 0
+
+            run_stats[v] = stat
+
+        return cls(**run_stats)
+
+
+class FrequencyDependentDielectric(BaseModel):
+    """Frequency-dependent dielectric data."""
+
+    real: List[List[float]] = Field(
+        None,
+        description="Real part of the frequency dependent dielectric constant, given at"
+        " each energy as 6 components according to XX, YY, ZZ, XY, YZ, ZX",
+    )
+    imaginary: List[List[float]] = Field(
+        None,
+        description="Imaginary part of the frequency dependent dielectric constant, "
+        "given at each energy as 6 components according to XX, YY, ZZ, XY, "
+        "YZ, ZX",
+    )
+    energy: List[float] = Field(
+        None,
+        description="Energies at which the real and imaginary parts of the dielectric"
+        "constant are given",
+    )
+
+    @classmethod
+    def from_vasprun(cls, vasprun: Vasprun) -> "FrequencyDependentDielectric":
+        """
+        Create a frequency-dependent dielectric calculation document from a vasprun.
+
+        Parameters
+        ----------
+        vasprun : Vasprun
+            A vasprun object.
+
+        Returns
+        -------
+        FrequencyDependentDielectric
+            A frequency-dependent dielectric document.
+        """
+        energy, real, imag = vasprun.dielectric
+        return cls(real=real, imaginary=imag, energy=energy)
+
+
+class ElectronPhononDisplacedStructures(BaseModel):
+    """Document defining electron phonon displaced structures."""
+
+    temperatures: List[float] = Field(
+        None,
+        description="The temperatures at which the electron phonon displacements "
+        "were generated.",
+    )
+    structures: List[Structure] = Field(
+        None, description="The displaced structures corresponding to each temperature."
+    )
 
 
 class CalculationOutput(BaseModel):
@@ -251,6 +317,11 @@ class CalculationOutput(BaseModel):
     epsilon_ionic: Matrix3D = Field(
         None, description="The ionic part of the dielectric constant"
     )
+    frequency_dependent_dielectric: FrequencyDependentDielectric = Field(
+        None,
+        description="Frequency-dependent dielectric information from an LOPTICS "
+        "calculation",
+    )
     ionic_steps: List[Dict[str, Any]] = Field(
         None, description="Energy, forces, and structure for each ionic step"
     )
@@ -274,13 +345,22 @@ class CalculationOutput(BaseModel):
     normalmode_eigenvecs: List[List[Vector3D]] = Field(
         None, description="Normal mode eigenvectors of phonon modes at Gamma"
     )
+    elph_displaced_structures: ElectronPhononDisplacedStructures = Field(
+        None,
+        description="Electron-phonon displaced structures, generated by setting "
+        "PHON_LMC = True.",
+    )
     run_stats: RunStatistics = Field(
         None, description="Summary of runtime statistics for this calculation"
     )
 
     @classmethod
     def from_vasp_outputs(
-        cls, vasprun: Vasprun, outcar: Outcar, locpot: Optional[Locpot] = None
+        cls,
+        vasprun: Vasprun,
+        outcar: Outcar,
+        locpot: Optional[Locpot] = None,
+        elph_poscars: Optional[List[Path]] = None,
     ) -> "CalculationOutput":
         """
         Create a VASP output document from VASP outputs.
@@ -299,7 +379,7 @@ class CalculationOutput(BaseModel):
             The VASP calculation output document.
         """
         try:
-            bandstructure = vasprun.get_band_structure()
+            bandstructure = vasprun.get_band_structure(efermi="smart")
             bandgap_info = bandstructure.get_band_gap()
             electronic_output = dict(
                 vbm=bandstructure.get_vbm()["energy"],
@@ -316,6 +396,12 @@ class CalculationOutput(BaseModel):
                 logger.warning("VASP doesn't properly output efermi for IBRION == 1")
             electronic_output = {}
 
+        freq_dependent_diel: Union[dict, FrequencyDependentDielectric] = {}
+        try:
+            freq_dependent_diel = FrequencyDependentDielectric.from_vasprun(vasprun)
+        except KeyError:
+            pass
+
         locpot_avg = None
         if locpot:
             locpot_avg = {
@@ -350,6 +436,14 @@ class CalculationOutput(BaseModel):
             magmoms = [m["tot"] for m in outcar.magnetization]
             structure.add_site_property("magmom", magmoms)
 
+        elph_structures: Dict[str, List[Any]] = {}
+        if elph_poscars is not None:
+            elph_structures.update({"temperatures": [], "structures": []})
+            for elph_poscar in elph_poscars:
+                temp = str(elph_poscar.name).replace("POSCAR.T=", "").replace(".gz", "")
+                elph_structures["temperatures"].append(temp)
+                elph_structures["structures"].append(Structure.from_file(elph_poscar))
+
         return cls(
             structure=structure,
             energy=vasprun.final_energy,
@@ -357,6 +451,8 @@ class CalculationOutput(BaseModel):
             epsilon_static=vasprun.epsilon_static or None,
             epsilon_static_wolfe=vasprun.epsilon_static_wolfe or None,
             epsilon_ionic=vasprun.epsilon_ionic or None,
+            frequency_dependent_dielectric=freq_dependent_diel,
+            elph_displaced_structures=elph_structures,
             ionic_steps=vasprun.ionic_steps,
             locpot=locpot_avg,
             outcar=outcar_dict,
@@ -409,11 +505,14 @@ class Calculation(BaseModel):
         task_name: str,
         vasprun_file: Union[Path, str],
         outcar_file: Union[Path, str],
-        volumetric_files: List[str],
+        volumetric_files: List[str] = None,
+        elph_poscars: List[Path] = None,
         parse_dos: Union[str, bool] = False,
         parse_bandstructure: Union[str, bool] = False,
         average_locpot: bool = True,
         run_bader: bool = SETTINGS.VASP_RUN_BADER and _BADER_EXE_EXISTS,
+        strip_bandstructure_projections: bool = False,
+        strip_dos_projections: bool = False,
         store_volumetric_data: Optional[
             Tuple[str]
         ] = SETTINGS.VASP_STORE_VOLUMETRIC_DATA,
@@ -434,6 +533,9 @@ class Calculation(BaseModel):
             Path to the OUTCAR file, relative to dir_name.
         volumetric_files
             Path to volumetric files, relative to dir_name.
+        elph_poscars
+            Path to displaced electron-phonon coupling POSCAR files generated using
+            ``PHON_LMC = True``, given relative to dir_name.
         parse_dos
             Whether to parse the DOS. Can be:
 
@@ -458,6 +560,12 @@ class Calculation(BaseModel):
             Whether to store the average of the LOCPOT along the crystal axes.
         run_bader
             Whether to run bader on the charge density.
+        strip_dos_projections : bool
+            Whether to strip the element and site projections from the density of states.
+            This can help reduce the size of DOS objects in systems with many atoms.
+        strip_bandstructure_projections : bool
+            Whether to strip the element and site projections from the band structure.
+            This can help reduce the size of DOS objects in systems with many atoms.
         store_volumetric_data
             Which volumetric files to store.
         vasprun_kwargs
@@ -473,6 +581,7 @@ class Calculation(BaseModel):
         outcar_file = dir_name / outcar_file
 
         vasprun_kwargs = vasprun_kwargs if vasprun_kwargs else {}
+        volumetric_files = [] if volumetric_files is None else volumetric_files
         vasprun = Vasprun(vasprun_file, **vasprun_kwargs)
         outcar = Outcar(outcar_file)
         completed_at = str(datetime.fromtimestamp(vasprun_file.stat().st_mtime))
@@ -484,10 +593,14 @@ class Calculation(BaseModel):
 
         dos = _parse_dos(parse_dos, vasprun)
         if dos is not None:
+            if strip_dos_projections:
+                dos = Dos(dos.efermi, dos.energies, dos.densities)
             vasp_objects[VaspObject.DOS] = dos  # type: ignore
 
         bandstructure = _parse_bandstructure(parse_bandstructure, vasprun)
         if bandstructure is not None:
+            if strip_bandstructure_projections:
+                bandstructure.projections = {}
             vasp_objects[VaspObject.BANDSTRUCTURE] = bandstructure  # type: ignore
 
         bader = None
@@ -504,7 +617,9 @@ class Calculation(BaseModel):
                 locpot = Locpot.from_file(dir_name / locpot_file)
 
         input_doc = CalculationInput.from_vasprun(vasprun)
-        output_doc = CalculationOutput.from_vasp_outputs(vasprun, outcar, locpot=locpot)
+        output_doc = CalculationOutput.from_vasp_outputs(
+            vasprun, outcar, locpot=locpot, elph_poscars=elph_poscars
+        )
 
         has_vasp_completed = Status.SUCCESS if vasprun.converged else Status.FAILED
         return (
@@ -591,7 +706,7 @@ def _get_volumetric_data(
     return volumetric_data
 
 
-def _parse_dos(parse_mode: Union[str, bool], vasprun: Vasprun) -> Optional[Dict]:
+def _parse_dos(parse_mode: Union[str, bool], vasprun: Vasprun) -> Optional[Dos]:
     """Parse DOS. See Calculation.from_vasp_files for supported arguments."""
     nsw = vasprun.incar.get("NSW", 0)
     dos = None
@@ -602,7 +717,7 @@ def _parse_dos(parse_mode: Union[str, bool], vasprun: Vasprun) -> Optional[Dict]
 
 def _parse_bandstructure(
     parse_mode: Union[str, bool], vasprun: Vasprun
-) -> Optional[Dict[str, Any]]:
+) -> Optional[BandStructure]:
     """Parse band structure. See Calculation.from_vasp_files for supported arguments."""
     vasprun_file = vasprun.filename
 
@@ -611,13 +726,15 @@ def _parse_bandstructure(
             # NSCF calculation
             bs_vrun = BSVasprun(vasprun_file, parse_projected_eigen=True)
             try:
-                bs = bs_vrun.get_band_structure(line_mode=True)  # try parsing line mode
+                # try parsing line mode
+                bs = bs_vrun.get_band_structure(line_mode=True, efermi="smart")
             except Exception:
-                bs = bs_vrun.get_band_structure()  # treat as a regular calculation
+                # treat as a regular calculation
+                bs = bs_vrun.get_band_structure(efermi="smart")
         else:
             # Not a NSCF calculation
             bs_vrun = BSVasprun(vasprun_file, parse_projected_eigen=False)
-            bs = bs_vrun.get_band_structure()
+            bs = bs_vrun.get_band_structure(efermi="smart")
 
         # only save the bandstructure if not moving ions
         if vasprun.incar.get("NSW", 0) <= 1:
@@ -626,7 +743,7 @@ def _parse_bandstructure(
     elif parse_mode:
         # legacy line/True behavior for bandstructure_mode
         bs_vrun = BSVasprun(vasprun_file, parse_projected_eigen=True)
-        bs = bs_vrun.get_band_structure(line_mode=(parse_mode == "line"))
+        bs = bs_vrun.get_band_structure(line_mode=parse_mode == "line", efermi="smart")
         return bs
 
     return None
