@@ -3,6 +3,7 @@
 import os
 import warnings
 from copy import deepcopy
+from dataclasses import dataclass, field
 from itertools import groupby
 from pathlib import Path
 from typing import Any, Dict, List, Optional, Union
@@ -12,6 +13,7 @@ from monty.io import zopen
 from monty.serialization import loadfn
 from pkg_resources import resource_filename
 from pymatgen.core import Structure
+from pymatgen.electronic_structure.core import Magmom
 from pymatgen.io.vasp import Incar, Kpoints, Outcar, Poscar, Potcar, Vasprun
 from pymatgen.io.vasp.sets import (
     BadInputSetWarning,
@@ -101,7 +103,11 @@ class VaspInputSet(InputSet):
         for k, v in inputs.items():
             if v is not None and (overwrite or not (directory / k).exists()):
                 with zopen(directory / k, "wt") as f:
-                    f.write(v.__str__())
+                    if isinstance(v, Poscar):
+                        # write POSCAR with more signifcant figures
+                        f.write(v.get_string(significant_figures=16))
+                    else:
+                        f.write(v.__str__())
             elif not overwrite and (directory / k).exists():
                 raise FileExistsError(f"{directory / k} already exists.")
 
@@ -167,9 +173,33 @@ class VaspInputSet(InputSet):
                 BadInputSetWarning,
             )
 
+        if self.incar.get("LHFCALC", False) is True and self.incar.get(
+            "ALGO", "Normal"
+        ) not in [
+            "Normal",
+            "All",
+            "Damped",
+        ]:
+            warnings.warn(
+                "Hybrid functionals only support Algo = All, Damped, or Normal.",
+                BadInputSetWarning,
+            )
+
+        if not self.incar.get("LASPH", False) and (
+            self.incar.get("METAGGA")
+            or self.incar.get("LHFCALC", False)
+            or self.incar.get("LDAU", False)
+            or self.incar.get("LUSE_VDW", False)
+        ):
+            warnings.warn(
+                "LASPH = True should be set for +U, meta-GGAs, hybrids, and vdW-DFT",
+                BadInputSetWarning,
+            )
+
         return True
 
 
+@dataclass
 class VaspInputSetGenerator(InputSetGenerator):
     """
     A class to generate VASP input sets.
@@ -203,7 +233,7 @@ class VaspInputSetGenerator(InputSetGenerator):
         when generating the input set from a previous VASP directory.
     constrain_total_magmom
         Whether to constrain the total magmom (NUPDOWN in INCAR) to be the sum of the
-        expected MAGMOM for all species.
+        initial MAGMOM guess for all species.
     validate_magmom
         Ensure that missing magmom values are filled in with the default value of 1.0.
     use_structure_charge
@@ -224,42 +254,32 @@ class VaspInputSetGenerator(InputSetGenerator):
         optB86b and rVV10.
     symprec
         Tolerance for symmetry finding, used for line mode band structure k-points.
+    auto_ispin
+        If generating input set from a previous calculation, this controls whether
+        to disable magnetisation (ISPIN = 1) if the absoluate valuue of all magnetic
+        moments are less than 0.02.
     config_dict
         The config dictionary to use containing the base input set settings.
     """
 
-    def __init__(
-        self,
-        user_incar_settings: Dict = None,
-        user_kpoints_settings: Union[Dict, Kpoints] = None,
-        user_potcar_settings: Dict = None,
-        user_potcar_functional: str = None,
-        auto_kspacing: bool = True,
-        constrain_total_magmom: bool = False,
-        validate_magmom: bool = True,
-        use_structure_charge: bool = False,
-        sort_structure: bool = True,
-        force_gamma: bool = True,
-        symprec: float = SETTINGS.SYMPREC,
-        vdw: str = None,
-        config_dict: Dict = None,
-    ):
-        if config_dict is None:
-            config_dict = _BASE_VASP_SET
+    user_incar_settings: Dict = field(default_factory=dict)
+    user_kpoints_settings: Union[Dict, Kpoints] = field(default_factory=dict)
+    user_potcar_settings: Dict = field(default_factory=dict)
+    user_potcar_functional: str = None
+    auto_kspacing: bool = True
+    constrain_total_magmom: bool = False
+    validate_magmom: bool = True
+    use_structure_charge: bool = False
+    sort_structure: bool = True
+    force_gamma: bool = True
+    symprec: float = SETTINGS.SYMPREC
+    vdw: str = None
+    auto_ispin: bool = False
+    config_dict: Dict = field(default_factory=lambda: _BASE_VASP_SET)
 
-        self._config_dict = deepcopy(config_dict)
-        self.user_incar_settings = user_incar_settings or {}
-        self.user_kpoints_settings: Union[Dict, Kpoints] = user_kpoints_settings or {}
-        self.user_potcar_settings = user_potcar_settings
-        self.user_potcar_functional = user_potcar_functional
-        self.auto_kspacing = auto_kspacing
-        self.constrain_total_magmom = constrain_total_magmom
-        self.validate_magmom = validate_magmom
-        self.sort_structure = sort_structure
-        self.force_gamma = force_gamma
-        self.symprec = symprec  # used in k-point generation
-        self.vdw = None if vdw is None else vdw.lower()
-        self.use_structure_charge = use_structure_charge
+    def __post_init__(self):
+        """Post init formatting of arguments."""
+        self.vdw = None if self.vdw is None else self.vdw.lower()
 
         if self.user_incar_settings.get("KSPACING") and self.user_kpoints_settings:
             warnings.warn(
@@ -280,13 +300,16 @@ class VaspInputSetGenerator(InputSetGenerator):
                     "Invalid or unsupported van-der-Waals functional. Supported "
                     f"functionals are {vdw_par.keys()}"
                 )
-            self._config_dict["INCAR"].update(vdw_par[self.vdw])
+            self.config_dict["INCAR"].update(vdw_par[self.vdw])
 
         # read the POTCAR_FUNCTIONAL from the .yaml
-        self.potcar_functional = self._config_dict.get("POTCAR_FUNCTIONAL", "PS")
+        self.potcar_functional = self.config_dict.get("POTCAR_FUNCTIONAL", "PS")
 
         # warn if a user is overriding POTCAR_FUNCTIONAL
-        if user_potcar_functional and user_potcar_functional != self.potcar_functional:
+        if (
+            self.user_potcar_functional
+            and self.user_potcar_functional != self.potcar_functional
+        ):
             warnings.warn(
                 "Overriding the POTCAR functional is generally not recommended "
                 "as it can significantly affect the results of calculations and "
@@ -295,7 +318,7 @@ class VaspInputSetGenerator(InputSetGenerator):
                 "not be available in the selected functional.",
                 BadInputSetWarning,
             )
-            self.potcar_functional = user_potcar_functional
+            self.potcar_functional = self.user_potcar_functional
 
         if self.user_potcar_settings:
             warnings.warn(
@@ -308,7 +331,7 @@ class VaspInputSetGenerator(InputSetGenerator):
                 BadInputSetWarning,
             )
             for k, v in self.user_potcar_settings.items():
-                self._config_dict["POTCAR"][k] = v
+                self.config_dict["POTCAR"][k] = v
 
     def get_input_set(  # type: ignore
         self,
@@ -340,7 +363,7 @@ class VaspInputSetGenerator(InputSetGenerator):
         VaspInputSet
             A VASP input set.
         """
-        structure, prev_incar, bandgap, vasprun, outcar = self._get_previous(
+        structure, prev_incar, bandgap, ispin, vasprun, outcar = self._get_previous(
             structure, prev_dir
         )
         incar_updates = self.get_incar_updates(
@@ -360,7 +383,12 @@ class VaspInputSetGenerator(InputSetGenerator):
         kspacing = self._kspacing(incar_updates)
         kpoints = self._get_kpoints(structure, kpoints_updates, kspacing)
         incar = self._get_incar(
-            structure, kpoints, prev_incar, incar_updates, bandgap=bandgap
+            structure,
+            kpoints,
+            prev_incar,
+            incar_updates,
+            bandgap=bandgap,
+            ispin=ispin,
         )
         return VaspInputSet(
             incar=incar,
@@ -453,7 +481,7 @@ class VaspInputSetGenerator(InputSetGenerator):
         nelect = sum(num_atoms * nelec[str(el)] for el, num_atoms in comp.items())
 
         if self.use_structure_charge:
-            return nelect - self.structure.charge
+            return nelect - structure.charge
 
         return nelect
 
@@ -469,19 +497,37 @@ class VaspInputSetGenerator(InputSetGenerator):
         vasprun = None
         outcar = None
         bandgap = 0
+        ispin = None
         if prev_dir:
             vasprun, outcar = get_vasprun_outcar(prev_dir)
-            bs = vasprun.get_band_structure()
+
+            if vasprun.efermi is None:
+                # VASP doesn't output efermi in vasprun if IBRION = 1
+                vasprun.efermi = outcar.efermi
+
+            bs = vasprun.get_band_structure(efermi="smart")
             prev_incar = vasprun.incar
             prev_structure = vasprun.final_structure
             bandgap = 0 if bs.is_metal() else bs.get_band_gap()["energy"]
 
+            if self.auto_ispin:
+                # turn off spin when magmom for every site is smaller than 0.02.
+                ispin = _get_ispin(vasprun, outcar)
+
         structure = structure if structure is not None else prev_structure
         structure = self._get_structure(structure)
-        return structure, prev_incar, bandgap, vasprun, outcar
+
+        return structure, prev_incar, bandgap, ispin, vasprun, outcar
 
     def _get_structure(self, structure):
         """Get the standardized structure."""
+        for site in structure:
+            if "magmom" in site.properties and isinstance(
+                site.properties["magmom"], Magmom
+            ):
+                # required to fix bug in get_valid_magmom_struct
+                site.properties["magmom"] = list(site.properties["magmom"])
+
         if self.sort_structure:
             structure = structure.get_sorted_structure()
 
@@ -492,7 +538,7 @@ class VaspInputSetGenerator(InputSetGenerator):
     def _get_potcar(self, structure, potcar_spec: bool = False):
         """Get the POTCAR."""
         elements = [a[0] for a in groupby([s.specie.symbol for s in structure])]
-        potcar_symbols = [self._config_dict["POTCAR"].get(el, el) for el in elements]
+        potcar_symbols = [self.config_dict["POTCAR"].get(el, el) for el in elements]
 
         if potcar_spec:
             return potcar_symbols
@@ -519,11 +565,12 @@ class VaspInputSetGenerator(InputSetGenerator):
         previous_incar: Dict = None,
         incar_updates: Dict = None,
         bandgap: float = 0.0,
+        ispin: int = None,
     ):
         """Get the INCAR."""
         previous_incar = {} if previous_incar is None else previous_incar
         incar_updates = {} if incar_updates is None else incar_updates
-        incar_settings = dict(self._config_dict["INCAR"])
+        incar_settings = dict(self.config_dict["INCAR"])
 
         # apply user incar settings to SETTINGS not to INCAR
         _apply_incar_updates(incar_settings, self.user_incar_settings)
@@ -549,6 +596,15 @@ class VaspInputSetGenerator(InputSetGenerator):
 
         if self.constrain_total_magmom:
             nupdown = sum(mag if abs(mag) > 0.6 else 0 for mag in incar["MAGMOM"])
+            if abs(nupdown - round(nupdown)) > 1e-5:
+                warnings.warn(
+                    "constrain_total_magmom was set to True, but the sum of MAGMOM "
+                    "values is not an integer. NUPDOWN is meant to set the spin "
+                    "multiplet and should typically be an integer. You are likely "
+                    "better off changing the values of MAGMOM or simply setting "
+                    "NUPDOWN directly in your INCAR settings.",
+                    UserWarning,
+                )
             incar["NUPDOWN"] = nupdown
 
         if self.use_structure_charge:
@@ -564,9 +620,16 @@ class VaspInputSetGenerator(InputSetGenerator):
             kpoints,
         )
 
+        # handle auto ISPIN
+        if ispin is not None and "ISPIN" not in self.user_incar_settings:
+            incar["ISPIN"] = ispin
+
         # apply specified updates, be careful not to override user_incar_settings
         _apply_incar_updates(incar, incar_updates, skip=self.user_incar_settings.keys())
 
+        # Remove unused INCAR parameters
+        _remove_unused_incar_params(incar, skip=self.user_incar_settings.keys())
+
         return incar
 
     def _get_kpoints(
@@ -583,7 +646,7 @@ class VaspInputSetGenerator(InputSetGenerator):
             kconfig = deepcopy(self.user_kpoints_settings)
         else:
             # apply updates to k-points config
-            kconfig = deepcopy(self._config_dict.get("KPOINTS", {}))
+            kconfig = deepcopy(self.config_dict.get("KPOINTS", {}))
             kconfig.update(kpoints_updates)
 
         # Return None if KSPACING is set and no other user k-points settings have been
@@ -613,7 +676,7 @@ class VaspInputSetGenerator(InputSetGenerator):
         base_kpoints = None
         if kconfig.get("line_density"):
             # handle line density generation
-            kpath = HighSymmKpath(structure)
+            kpath = HighSymmKpath(structure, **kconfig.get("kpath_kwargs", {}))
             frac_k_points, k_points_labels = kpath.get_kpoints(
                 line_density=kconfig["line_density"], coords_are_cartesian=False
             )
@@ -677,7 +740,7 @@ class VaspInputSetGenerator(InputSetGenerator):
                 style=Kpoints.supported_modes.Reciprocal,
                 num_kpts=len(mesh),
                 kpts=[i[0] for i in mesh],
-                kpts_weights=[i[1] for i in mesh],
+                kpts_weights=[0 for i in mesh],
             )
 
         added_kpoints = None
@@ -688,7 +751,7 @@ class VaspInputSetGenerator(InputSetGenerator):
                 num_kpts=len(kconfig.get("added_kpoints")),
                 kpts=kconfig.get("added_kpoints"),
                 labels=["user-defined"] * len(kconfig.get("added_kpoints")),
-                kpts_weights=[1] * len(kconfig.get("added_kpoints")),
+                kpts_weights=[0] * len(kconfig.get("added_kpoints")),
             )
 
         if base_kpoints and not (added_kpoints or zero_weighted_kpoints):
@@ -725,8 +788,8 @@ class VaspInputSetGenerator(InputSetGenerator):
             return self.user_incar_settings["KSPACING"]
         if "KSPACING" in incar_updates:
             return incar_updates["KSPACING"]
-        if "KSPACING" in self._config_dict["INCAR"]:
-            return self._config_dict["INCAR"]["KSPACING"]
+        if "KSPACING" in self.config_dict["INCAR"]:
+            return self.config_dict["INCAR"]["KSPACING"]
         return None
 
 
@@ -735,7 +798,7 @@ def _get_kspacing(bandgap: float) -> float:
     if bandgap == 0:
         return 0.22
 
-    rmin = 25.22 - 2.87 * bandgap  # Eq. 25
+    rmin = max(1.5, 25.22 - 2.87 * bandgap)  # Eq. 25
     kspacing = 2 * np.pi * 1.0265 / (rmin - 1.0183)  # Eq. 29
 
     # cap kspacing at a max of 0.44, per internal benchmarking
@@ -751,18 +814,18 @@ def _get_magmoms(magmoms, structure):
         elif hasattr(site.specie, "spin"):
             mag.append(site.specie.spin)
         elif str(site.specie) in magmoms:
-            if site.specie.symbol == "Co":
+            if site.specie.symbol == "Co" and magmoms[str(site.specie)] <= 1.0:
                 warnings.warn(
-                    "Co without oxidation state is initialized low spin by default. If "
-                    "this is not desired, please set the spin on the magmom on the "
+                    "Co without an oxidation state is initialized as low spin by default in Atomate2. "
+                    "If this default behavior is not desired, please set the spin on the magmom on the "
                     "site directly to ensure correct initialization."
                 )
             mag.append(magmoms.get(str(site.specie)))
         else:
             if site.specie.symbol == "Co":
                 warnings.warn(
-                    "Co without oxidation state is initialized low spin by default. If "
-                    "this is not desired, please set the spin on the magmom on the "
+                    "Co without an oxidation state is initialized as low spin by default in Atomate2. "
+                    "If this default behavior is not desired, please set the spin on the magmom on the "
                     "site directly to ensure correct initialization."
                 )
             mag.append(magmoms.get(site.specie.symbol, 0.6))
@@ -803,23 +866,25 @@ def _set_u_params(incar, incar_settings, structure):
     """Modify INCAR for use with U parameters."""
     has_u = incar_settings.get("LDAU", False) and sum(incar["LDAUU"]) > 0
 
-    if has_u:
-        # modify LMAXMIX if LSDA+U and you have d or f electrons
-        # note that if the user explicitly sets LMAXMIX in settings it will
-        # override this logic.
-        if "LMAXMIX" not in incar_settings.keys():
-            # contains f-electrons
-            if any(el.Z > 56 for el in structure.composition):
-                incar["LMAXMIX"] = 6
-            # contains d-electrons
-            elif any(el.Z > 20 for el in structure.composition):
-                incar["LMAXMIX"] = 4
-
-    else:
+    if not has_u:
         for key in list(incar.keys()):
             if key.startswith("LDAU"):
                 del incar[key]
 
+    # Modify LMAXMIX if you have d or f electrons present. Note that if the user
+    # explicitly sets LMAXMIX in settings it will override this logic. Previously, this
+    # was only set if Hubbard U was enabled as per the VASP manual but following an
+    # investigation it was determined that this would lead to a significant difference
+    # between SCF -> NonSCF even without Hubbard U enabled. Thanks to Andrew Rosen for
+    # investigating and reporting.
+    if "LMAXMIX" not in incar_settings.keys():
+        # contains f-electrons
+        if any(el.Z > 56 for el in structure.composition):
+            incar["LMAXMIX"] = 6
+        # contains d-electrons
+        elif any(el.Z > 20 for el in structure.composition):
+            incar["LMAXMIX"] = 4
+
 
 def _apply_incar_updates(incar, updates, skip=None):
     """
@@ -845,6 +910,38 @@ def _apply_incar_updates(incar, updates, skip=None):
             incar[k] = v
 
 
+def _remove_unused_incar_params(incar, skip=None):
+    """
+    Remove INCAR parameters that are not actively used by VASP.
+
+    Parameters
+    ----------
+    incar
+        An incar.
+    skip
+        Keys to skip.
+    """
+    skip = () if skip is None else skip
+
+    # Turn off IBRION/ISIF/POTIM if NSW = 0
+    opt_flags = ["EDIFFG", "IBRION", "ISIF", "POTIM"]
+    if incar.get("NSW", 0) == 0:
+        for opt_flag in opt_flags:
+            if opt_flag not in skip:
+                incar.pop(opt_flag, None)
+
+    # Remove MAGMOMs if they aren't used
+    if incar.get("ISPIN", 1) == 1 and "MAGMOM" not in skip:
+        incar.pop("MAGMOM", None)
+
+    # Turn off +U flags if +U is not even used
+    ldau_flags = ["LDAUU", "LDAUJ", "LDAUL", "LDAUTYPE"]
+    if incar.get("LDAU", False) is False:
+        for ldau_flag in ldau_flags:
+            if ldau_flag not in skip:
+                incar.pop(ldau_flag, None)
+
+
 def _set_kspacing(
     incar, incar_settings, user_incar_settings, auto_kspacing, bandgap, kpoints
 ):
@@ -916,3 +1013,14 @@ def _combine_kpoints(*kpoints_objects: Kpoints):
         labels=labels,
         kpts_weights=weights,
     )
+
+
+def _get_ispin(vasprun: Optional[Vasprun], outcar: Optional[Outcar]):
+    """Get value of ISPIN depending on the magnetisation in the OUTCAR and vasprun."""
+    if outcar is not None and outcar.magnetization is not None:
+        # Turn off spin when magmom for every site is smaller than 0.02.
+        site_magmom = np.array([i["tot"] for i in outcar.magnetization])
+        return 2 if np.any(np.abs(site_magmom) > 0.02) else 1
+    elif vasprun is not None:
+        return 2 if vasprun.is_spin else 1
+    return 2
