@@ -1,7 +1,8 @@
 """Module defining core VASP input set generators."""
 
 from copy import deepcopy
-from typing import Any, Dict, List, Optional
+from dataclasses import dataclass, field
+from typing import Any, Dict, List, Optional, Tuple
 
 import numpy as np
 from pymatgen.core import Structure
@@ -19,9 +20,11 @@ __all__ = [
     "HSEStaticSetGenerator",
     "HSEBSSetGenerator",
     "HSETightRelaxSetGenerator",
+    "ElectronPhononSetGenerator",
 ]
 
 
+@dataclass
 class RelaxSetGenerator(VaspInputSetGenerator):
     """Class to generate VASP relaxation input sets."""
 
@@ -57,6 +60,7 @@ class RelaxSetGenerator(VaspInputSetGenerator):
         return {"NSW": 99, "LCHARG": False, "ISIF": 3, "IBRION": 2}
 
 
+@dataclass
 class TightRelaxSetGenerator(VaspInputSetGenerator):
     """Class to generate tight VASP relaxation input sets."""
 
@@ -97,12 +101,12 @@ class TightRelaxSetGenerator(VaspInputSetGenerator):
             "LAECHG": False,
             "EDIFFG": -0.001,
             "LREAL": False,
-            "ALGO": "Normal",
             "NSW": 99,
             "LCHARG": False,
         }
 
 
+@dataclass
 class StaticSetGenerator(VaspInputSetGenerator):
     """
     Class to generate VASP static input sets.
@@ -119,10 +123,8 @@ class StaticSetGenerator(VaspInputSetGenerator):
         Other keyword arguments that will be passed to :obj:`VaspInputSetGenerator`.
     """
 
-    def __init__(self, lepsilon: bool = False, lcalcpol: bool = False, **kwargs):
-        super().__init__(**kwargs)
-        self.lepsilon = lepsilon
-        self.lcalcpol = lcalcpol
+    lepsilon: bool = False
+    lcalcpol: bool = False
 
     def get_incar_updates(
         self,
@@ -154,23 +156,24 @@ class StaticSetGenerator(VaspInputSetGenerator):
             A dictionary of updates to apply.
         """
         updates = {
-            "NSW": 1,
+            "NSW": 0,
             "ISMEAR": -5,
             "LCHARG": True,
             "LORBIT": 11,
-            "ALGO": "Normal",
+            "LREAL": False,
         }
         if self.lepsilon:
             # LPEAD=T: numerical evaluation of overlap integral prevents LRF_COMMUTATOR
             # errors and can lead to better expt. agreement but produces slightly
             # different results
-            updates.update({"IBRION": 8, "LEPSILON": True, "LPEAD": True})
+            updates.update({"IBRION": 8, "LEPSILON": True, "LPEAD": True, "NSW": 1})
 
         if self.lcalcpol:
             updates["LCALCPOL"] = True
         return updates
 
 
+@dataclass
 class NonSCFSetGenerator(VaspInputSetGenerator):
     """
     Class to generate VASP non-self-consistent field input sets.
@@ -194,23 +197,18 @@ class NonSCFSetGenerator(VaspInputSetGenerator):
         Other keyword arguments that will be passed to :obj:`VaspInputSetGenerator`.
     """
 
-    def __init__(
-        self,
-        mode: str = "line",
-        dedos: float = 0.005,
-        reciprocal_density=100,
-        line_density=20,
-        optics: bool = False,
-        nbands_factor: float = 1.2,
-        **kwargs,
-    ):
-        super().__init__(**kwargs)
-        self.mode = mode.lower()
-        self.dedos = dedos
-        self.line_density = line_density
-        self.reciprocal_density = reciprocal_density
-        self.optics = optics
-        self.nbands_factor = nbands_factor
+    mode: str = "line"
+    dedos: float = 0.02
+    reciprocal_density: float = 100
+    line_density: float = 20
+    optics: bool = False
+    nbands_factor: float = 1.2
+    auto_ispin: bool = True
+
+    def __post_init__(self):
+        """Ensure mode is set correctly."""
+        super().__post_init__()
+        self.mode = self.mode.lower()
 
         supported_modes = ("line", "uniform", "boltztrap")
         if self.mode not in supported_modes:
@@ -285,7 +283,6 @@ class NonSCFSetGenerator(VaspInputSetGenerator):
             A dictionary of updates to apply.
         """
         updates: Dict[str, Any] = {
-            "IBRION": -1,
             "LCHARG": False,
             "LORBIT": 11,
             "LWAVE": False,
@@ -295,9 +292,6 @@ class NonSCFSetGenerator(VaspInputSetGenerator):
             "KSPACING": None,
         }
 
-        # turn off spin when magmom for every site is smaller than 0.02.
-        updates["ISPIN"] = _get_ispin(vasprun, outcar)
-
         if vasprun is not None:
             # set nbands
             nbands = int(np.ceil(vasprun.parameters["NBANDS"] * self.nbands_factor))
@@ -316,13 +310,15 @@ class NonSCFSetGenerator(VaspInputSetGenerator):
             updates.update({"ISMEAR": 0, "SIGMA": 0.01})
 
         if self.optics:
-            updates["LOPTICS"] = True
+            # LREAL not supported with LOPTICS = True
+            updates.update({"LOPTICS": True, "LREAL": False, "CSHIFT": 1e-5})
 
         updates["MAGMOM"] = None
 
         return updates
 
 
+@dataclass
 class HSERelaxSetGenerator(VaspInputSetGenerator):
     """Class to generate VASP HSE06 relaxation input sets."""
 
@@ -364,9 +360,11 @@ class HSERelaxSetGenerator(VaspInputSetGenerator):
             "IBRION": 2,
             "PRECFOCK": "Fast",
             "ISIF": 3,
+            "LASPH": True,
         }
 
 
+@dataclass
 class HSETightRelaxSetGenerator(VaspInputSetGenerator):
     """Class to generate tight VASP HSE relaxation input sets."""
 
@@ -414,9 +412,11 @@ class HSETightRelaxSetGenerator(VaspInputSetGenerator):
             "HFSCREEN": 0.2,
             "LHFCALC": True,
             "PRECFOCK": "Fast",
+            "LASPH": True,
         }
 
 
+@dataclass
 class HSEStaticSetGenerator(VaspInputSetGenerator):
     """Class to generate VASP HSE06 static input sets."""
 
@@ -450,19 +450,21 @@ class HSEStaticSetGenerator(VaspInputSetGenerator):
             A dictionary of updates to apply.
         """
         return {
-            "NSW": 1,
+            "NSW": 0,
             "ALGO": "All",
             "GGA": "PE",
             "HFSCREEN": 0.2,
             "LHFCALC": True,
-            "IBRION": 2,
             "PRECFOCK": "Fast",
             "ISMEAR": -5,
             "LORBIT": 11,
             "LCHARG": True,
+            "LASPH": True,
+            "LREAL": False,
         }
 
 
+@dataclass
 class HSEBSSetGenerator(VaspInputSetGenerator):
     """
     Class to generate VASP HSE06 band structure input sets.
@@ -509,28 +511,21 @@ class HSEBSSetGenerator(VaspInputSetGenerator):
         Other keyword arguments that will be passed to :obj:`VaspInputSetGenerator`.
     """
 
-    def __init__(
-        self,
-        mode: str = "gap",
-        dedos: float = 0.005,
-        reciprocal_density=50,
-        line_density=20,
-        zero_weighted_reciprocal_density=100,
-        optics: bool = False,
-        nbands_factor: float = 1.2,
-        added_kpoints: List[Vector3D] = None,
-        **kwargs,
-    ):
-        super().__init__(**kwargs)
-        self.mode = mode.lower()
-        self.dedos = dedos
-        self.line_density = line_density
-        self.reciprocal_density = reciprocal_density
-        self.optics = optics
-        self.nbands_factor = nbands_factor
-        self.added_kpoints = [] if added_kpoints is None else added_kpoints
-        self.zero_weighted_reciprocal_density = zero_weighted_reciprocal_density
+    mode: str = "gap"
+    dedos: float = 0.02
+    reciprocal_density: float = 50
+    line_density: float = 20
+    zero_weighted_reciprocal_density: float = 100
+    optics: bool = False
+    nbands_factor: float = 1.2
+    added_kpoints: List[Vector3D] = field(default_factory=list)
+    auto_ispin: bool = True
 
+    def __post_init__(self):
+        """Ensure mode is set correctly."""
+        super().__post_init__()
+
+        self.mode = self.mode.lower()
         supported_modes = ("line", "uniform", "gap", "uniform_dense")
         if self.mode not in supported_modes:
             raise ValueError(f"Supported modes are: {', '.join(supported_modes)}")
@@ -622,15 +617,15 @@ class HSEBSSetGenerator(VaspInputSetGenerator):
             "ALGO": "All",
             "GGA": "PE",
             "HFSCREEN": 0.2,
+            "PRECFOCK": "Fast",
             "LHFCALC": True,
             "LCHARG": False,
             "NELMIN": 5,
             "KSPACING": None,
+            "LORBIT": 11,
+            "LREAL": False,
         }
 
-        # turn off spin when magmom for every site is smaller than 0.02.
-        updates["ISPIN"] = _get_ispin(vasprun, outcar)
-
         if self.mode == "uniform" and len(self.added_kpoints) == 0:
             # automatic setting of nedos using the energy range and the energy step
             nedos = _get_nedos(vasprun, self.dedos)
@@ -649,13 +644,130 @@ class HSEBSSetGenerator(VaspInputSetGenerator):
             updates["NBANDS"] = nbands
 
         if self.optics:
-            updates["LOPTICS"] = True
+            # LREAL not supported with LOPTICS
+            updates.update({"LOPTICS": True, "LREAL": False, "CSHIFT": 1e-5})
 
         updates["MAGMOM"] = None
 
         return updates
 
 
+@dataclass
+class ElectronPhononSetGenerator(VaspInputSetGenerator):
+    """
+    Class to generate VASP electron phonon input sets.
+
+    .. note::
+        Requires VASP 6.0 and higher. See https://www.vasp.at/wiki/index.php/Electron-
+        phonon_interactions_from_Monte-Carlo_sampling for more details.
+
+    Parameters
+    ----------
+    temperatures : list of float
+        The temperatures for which the electron-phonon interactions are evaluated.
+    reciprocal_density
+        Density of k-mesh by reciprocal volume.
+    """
+
+    temperatures: Tuple[float, ...] = (
+        0,
+        100,
+        200,
+        300,
+        400,
+        500,
+        600,
+        700,
+        800,
+        900,
+        1000,
+    )
+    reciprocal_density: float = 50
+    auto_ispin: bool = True
+
+    def get_incar_updates(
+        self,
+        structure: Structure,
+        prev_incar: dict = None,
+        bandgap: float = 0,
+        vasprun: Vasprun = None,
+        outcar: Outcar = None,
+    ) -> dict:
+        """
+        Get updates to the INCAR for a static VASP job.
+
+        Parameters
+        ----------
+        structure
+            A structure.
+        prev_incar
+            An incar from a previous calculation.
+        bandgap
+            The band gap.
+        vasprun
+            A vasprun from a previous calculation.
+        outcar
+            An outcar from a previous calculation.
+
+        Returns
+        -------
+        dict
+            A dictionary of updates to apply.
+        """
+        return {
+            "NSW": 1,
+            "ISMEAR": 0,
+            "IBRION": 6,
+            "ISIF": 2,
+            "ENCUT": 700,
+            "EDIFF": 1e-7,
+            "LAECHG": False,
+            "LREAL": False,
+            "LCHARG": False,
+            "LVTOT": False,
+            "LVHAR": False,
+            "PREC": "Accurate",
+            "KSPACING": None,
+            "PHON_NTLIST": len(self.temperatures),
+            "PHON_TLIST": list(self.temperatures),  # has to be a list otherwise error
+            "PHON_NSTRUCT": 0,
+            "PHON_LMC": True,
+        }
+
+    def get_kpoints_updates(
+        self,
+        structure: Structure,
+        prev_incar: dict = None,
+        bandgap: float = 0.0,
+        vasprun: Vasprun = None,
+        outcar: Outcar = None,
+    ) -> dict:
+        """
+        Get updates to the kpoints configuration for a non-self consistent VASP job.
+
+        Note, these updates will be ignored if the user has set user_kpoint_settings.
+
+        Parameters
+        ----------
+        structure
+            A structure.
+        prev_incar
+            An incar from a previous calculation.
+        bandgap
+            The band gap.
+        vasprun
+            A vasprun from a previous calculation.
+        outcar
+            An outcar from a previous calculation.
+
+        Returns
+        -------
+        dict
+            A dictionary of updates to apply to the KPOINTS config.
+        """
+        return {"reciprocal_density": self.reciprocal_density}
+
+
 def _get_nedos(vasprun: Optional[Vasprun], dedos: float):
     """Automatic setting of nedos using the energy range and the energy step."""
     if vasprun is None:
@@ -664,14 +776,3 @@ def _get_nedos(vasprun: Optional[Vasprun], dedos: float):
     emax = max(eigs.max() for eigs in vasprun.eigenvalues.values())
     emin = min(eigs.min() for eigs in vasprun.eigenvalues.values())
     return int((emax - emin) / dedos)
-
-
-def _get_ispin(vasprun: Optional[Vasprun], outcar: Optional[Outcar]):
-    """Get value of ISPIN depending on the magnetisation in the OUTCAR and vasprun."""
-    if outcar is not None and outcar.magnetization is not None:
-        # Turn off spin when magmom for every site is smaller than 0.02.
-        site_magmom = np.array([i["tot"] for i in outcar.magnetization])
-        return 2 if np.any(np.abs(site_magmom) > 0.02) else 1
-    elif vasprun is not None:
-        return 2 if vasprun.is_spin else 1
-    return 2
