@@ -5,7 +5,10 @@ def test_elastic(mock_vasp, clean_dir, si_structure):
 
     from atomate2.common.schemas.elastic import ElasticDocument
     from atomate2.vasp.flows.elastic import ElasticMaker
-    from atomate2.vasp.powerups import update_user_kpoints_settings
+    from atomate2.vasp.powerups import (
+        update_user_incar_settings,
+        update_user_kpoints_settings,
+    )
 
     # mapping from job name to directory containing test files
     ref_paths = {
@@ -21,12 +24,12 @@ def test_elastic(mock_vasp, clean_dir, si_structure):
 
     # settings passed to fake_run_vasp; adjust these to check for certain INCAR settings
     fake_run_vasp_kwargs = {
-        "elastic relax 1/6": {"incar_settings": ["NSW", "ISMEAR"]},
-        "elastic relax 2/6": {"incar_settings": ["NSW", "ISMEAR"]},
-        "elastic relax 3/6": {"incar_settings": ["NSW", "ISMEAR"]},
-        "elastic relax 4/6": {"incar_settings": ["NSW", "ISMEAR"]},
-        "elastic relax 5/6": {"incar_settings": ["NSW", "ISMEAR"]},
-        "elastic relax 6/6": {"incar_settings": ["NSW", "ISMEAR"]},
+        "elastic relax 1/6": {"incar_settings": ["NSW", "ISMEAR", "ISIF"]},
+        "elastic relax 2/6": {"incar_settings": ["NSW", "ISMEAR", "ISIF"]},
+        "elastic relax 3/6": {"incar_settings": ["NSW", "ISMEAR", "ISIF"]},
+        "elastic relax 4/6": {"incar_settings": ["NSW", "ISMEAR", "ISIF"]},
+        "elastic relax 5/6": {"incar_settings": ["NSW", "ISMEAR", "ISIF"]},
+        "elastic relax 6/6": {"incar_settings": ["NSW", "ISMEAR", "ISIF"]},
         "tight relax 1": {"incar_settings": ["NSW", "ISMEAR"]},
         "tight relax 2": {"incar_settings": ["NSW", "ISMEAR"]},
     }
@@ -37,7 +40,10 @@ def test_elastic(mock_vasp, clean_dir, si_structure):
     # generate flow
     si_prim = SpacegroupAnalyzer(si_structure).get_primitive_standard_structure()
     flow = ElasticMaker().make(si_prim)
-    update_user_kpoints_settings(flow, {"grid_density": 100}, name_filter="relax")
+    flow = update_user_kpoints_settings(
+        flow, {"grid_density": 100}, name_filter="relax"
+    )
+    flow = update_user_incar_settings(flow, {"KSPACING": None}, name_filter="relax")
 
     # Run the flow or job and ensure that it finished running successfully
     responses = run_locally(flow, create_folders=True, ensure_success=True)
